#include "../global.h"
#include "crrc_gui_main.h"
#include "crrc_ctrlgen.h"
#include "crrc_msgbox.h"
#include "crrc_calibmap.h"
#include "crrc_mousebutton.h"
#include "crrc_scaleinput.h"
#include "crrc_joy.h"
#include "util.h"

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <vector>

#include "../crrc_main.h"
#include "../crrc_main_cpp.h"
#include "../mouse_kbd.h"
#include "../lib_conversions.h"


static void CGUICtrlGeneralCallback(puObject *obj);
static void CGUIInputMethodCallback(puObject *obj);
static void CGUICalibrateButtonCallback(puObject *obj);
static void CGUIJoystickButtonCallback(puObject *obj);
static void CGUIMixerButtonCallback(puObject *obj);
static void CGUIDeviceCallback(puObject *obj);
static void CGUISpeedCallback(puObject *obj);
static void CGUIJoystickCallback(puObject *obj);

static char      *inputMethods[]    = {(char*)InputMethodStrings[0], (char*)InputMethodStrings[1],
                                       (char*)InputMethodStrings[2], (char*)InputMethodStrings[3],
                                       (char*)InputMethodStrings[4], (char*)InputMethodStrings[5],
                                       (char*)InputMethodStrings[6], (char*)InputMethodStrings[7],
                                       (char*)InputMethodStrings[8], (char*)InputMethodStrings[9],
                                       NULL};

// Order according to values of MOUSE and KEYBOARD!
static char      *zoomCtrl[]        = {"Keyboard", "Mouse", NULL};

static char      *radioTypes[]      = {(char*)RadioTypeStrings[0], (char*)RadioTypeStrings[1],
                                       (char*)RadioTypeStrings[2], (char*)RadioTypeStrings[3],
                                       (char*)RadioTypeStrings[4], (char*)RadioTypeStrings[5],
                                       NULL};

#ifdef WIN32
static char *serialDevs[] = {"COM1:", "COM2:", "COM3:", "COM4:", NULL};
#else
static char *serialDevs[] = {"/dev/ttyS0", "/dev/ttyS1", "/dev/ttyS2", "/dev/ttyS3", NULL};
#endif

static char *serialSpeed[] = {" 9600 / 0xF0", "19200 / 0xFF", NULL};

#define BUTTON_BOX_HEIGHT   (2*DLG_DEF_SPACE+DLG_DEF_BUTTON_HEIGHT)
#define COMBO_W  (150)
#define COMBO_H   (20)
#define LABEL_W  (140)


CGUICtrlGeneralDialog::CGUICtrlGeneralDialog()
            : CRRCDialog(0, 0, CRRC_DIALOG_OK)
{
  
  combo_inputMethod = new puComboBox( LABEL_W + DLG_DEF_SPACE, 
                                      BUTTON_BOX_HEIGHT + 7*DLG_DEF_SPACE + 4*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                      LABEL_W + COMBO_W,
                                      BUTTON_BOX_HEIGHT + 7*DLG_DEF_SPACE + 5*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                      inputMethods, false);
  combo_inputMethod->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_inputMethod->setLabel("Input Method");  
  combo_inputMethod->setCurrentItem(TXInterface->inputMethod());
  combo_inputMethod->setCallback(CGUIInputMethodCallback);
  combo_inputMethod->setUserData(this);
  combo_inputMethod->reveal();
  combo_inputMethod->activate();

  combo_joyNum = new puComboBox( LABEL_W + DLG_DEF_SPACE, 
                                 BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 3*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                 LABEL_W + COMBO_W,
                                 BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 4*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                 NULL, false);
  combo_joyNum->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_joyNum->setLabel("Joystick device");
  combo_joyNum->setCallback(CGUIJoystickCallback);
  combo_joyNum_list = NULL;
  nCombo_joyNum = 0;
  rebuildJoystickComboBox();

  combo_device = new puComboBox( LABEL_W + DLG_DEF_SPACE,
                                 BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 3*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                 LABEL_W + COMBO_W,
                                 BUTTON_BOX_HEIGHT + 6*DLG_DEF_SPACE + 4*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                 serialDevs, false);
  combo_device->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_device->setLabel("Serial port");
  combo_device->setCurrentItem(TXInterface->getDeviceNumber());
  combo_device->setCallback(CGUIDeviceCallback);
  combo_device->setUserData(this);
  combo_device->hide();

  combo_speed = new puComboBox( LABEL_W + DLG_DEF_SPACE,
                                BUTTON_BOX_HEIGHT + 5*DLG_DEF_SPACE + 2*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                LABEL_W + COMBO_W,
                                BUTTON_BOX_HEIGHT + 5*DLG_DEF_SPACE + 3*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT,
                                serialSpeed, false);
  combo_speed->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_speed->setLabel("Baudrate / Sync");
  combo_speed->setCurrentItem(TXInterface->getDeviceSpeed());
  combo_speed->setCallback(CGUISpeedCallback);
  combo_speed->setUserData(this);
  combo_speed->hide();

  button_calibrate = new puOneShot(LABEL_W + DLG_DEF_SPACE,
                                   BUTTON_BOX_HEIGHT + 4*DLG_DEF_SPACE + 2*COMBO_H + 2*DLG_DEF_BUTTON_HEIGHT,
                                   LABEL_W + COMBO_W,
                                   BUTTON_BOX_HEIGHT + 4*DLG_DEF_SPACE + 2*COMBO_H + 3*DLG_DEF_BUTTON_HEIGHT);
  button_calibrate->setLegend("Configure...");
  button_calibrate->setUserData(this);
  button_calibrate->greyOut();
  button_calibrate->setCallback(CGUICalibrateButtonCallback);

  button_mixer = new puOneShot(LABEL_W + DLG_DEF_SPACE,
                               BUTTON_BOX_HEIGHT + 3*DLG_DEF_SPACE + 2*COMBO_H + 1*DLG_DEF_BUTTON_HEIGHT,
                               LABEL_W + COMBO_W,
                               BUTTON_BOX_HEIGHT + 3*DLG_DEF_SPACE + 2*COMBO_H + 2*DLG_DEF_BUTTON_HEIGHT);
  button_mixer->setLegend("Mixer...");
  button_mixer->setUserData(this);
  button_mixer->greyOut();
  button_mixer->setCallback(CGUIMixerButtonCallback);

  button_joybuttons = new puOneShot(LABEL_W + DLG_DEF_SPACE,
                                   BUTTON_BOX_HEIGHT + 2*DLG_DEF_SPACE + 2*COMBO_H,
                                   LABEL_W + COMBO_W,
                                   BUTTON_BOX_HEIGHT + 2*DLG_DEF_SPACE + 2*COMBO_H + DLG_DEF_BUTTON_HEIGHT);
  button_joybuttons->setLegend("Buttons...");
  button_joybuttons->setUserData(this);
  button_joybuttons->greyOut();
  button_joybuttons->setCallback(CGUIJoystickButtonCallback);

  combo_zoom = new puComboBox( LABEL_W + DLG_DEF_SPACE,
                               BUTTON_BOX_HEIGHT + 0*DLG_DEF_SPACE + 0*COMBO_H,
                               LABEL_W + COMBO_W,
                               BUTTON_BOX_HEIGHT + 0*DLG_DEF_SPACE + 1*COMBO_H,
                               zoomCtrl, false);
  combo_zoom->setLabelPlace(PUPLACE_LOWER_LEFT);
  combo_zoom->setLabel("Zoom Control");
  combo_zoom->setCurrentItem(inputDev.zoom_control-1);
  combo_zoom->reveal();
  combo_zoom->activate();

  close();
  setSize(COMBO_W + LABEL_W + 2*DLG_DEF_SPACE, 
          3*DLG_DEF_BUTTON_HEIGHT + 5*COMBO_H + 8*DLG_DEF_SPACE + BUTTON_BOX_HEIGHT);
  setCallback(CGUICtrlGeneralCallback);
  
  // center the dialog on screen: doesn't work well, as the combo box 
  // when expanded upwards, seems to influence getABox -- but also getBBox,
  // so using that doesn't help.
  int wwidth, wheight;
  int current_width = getABox()->max[0] - getABox()->min[0];
  int current_height = getABox()->max[1] - getABox()->min[1];
  puGetWindowSize(&wwidth, &wheight);
  setPosition(wwidth/2 - current_width/2, wheight/2 - current_height/2);

  reveal();
  
  // show/hide widgets
  CGUIInputMethodCallback((puObject*)combo_inputMethod);
}


/**
 * Destroy the dialog.
 */

CGUICtrlGeneralDialog::~CGUICtrlGeneralDialog()
{
  if (combo_joyNum_list != NULL)
  {
    T_GUI_Util::freenames(combo_joyNum_list, nCombo_joyNum);
    combo_joyNum_list = NULL;
    nCombo_joyNum = 0;
  }
}


/**
 * Rebuild the list behind the joystick selection combo box
 */
void CGUICtrlGeneralDialog::rebuildJoystickComboBox()
{
  std::vector<std::string> joynames;
  
  #if TEST_WITHOUT_JOYSTICK > 0
  for (int i = 0; i < TEST_WITHOUT_JOYSTICK; i++)
  {
    char number[7];
    std::string name;
    
    snprintf(number, 6, "%2d - ", i);
    name = number;
    name += "fake joystick";
    joynames.push_back(name);
  }
  #else
  for (int i = 0; i < SDL_NumJoysticks(); i++)
  {
    char number[7];
    std::string name;
    
    snprintf(number, 6, "%2d - ", i);
    name = number;
    if (SDL_JoystickName(i) != NULL)
    {
      name += SDL_JoystickName(i);
    }
    else
    {
      name += "not available (removed?)";
      std::cerr << "CGUICtrlGeneralDialog::rebuildJoystickComboBox(): error:" << std::endl;
      std::cerr << "  Joystick " << i << " is no longer available (removed after starting CRRCsim?)." << std::endl;
    }
    joynames.push_back(name);
  }
  #endif
  
  if (combo_joyNum_list != NULL)
  {
    T_GUI_Util::freenames(combo_joyNum_list, nCombo_joyNum);
    combo_joyNum_list = NULL;
    nCombo_joyNum = 0;
  }
  combo_joyNum_list = T_GUI_Util::loadnames(joynames, nCombo_joyNum);
  combo_joyNum->newList(combo_joyNum_list);
}


/** \brief The dialog's callback.
 *
 */
void CGUICtrlGeneralCallback(puObject *obj)
{
  if (obj->getIntegerValue() == CRRC_DIALOG_OK)
  {    
    // Dialog left by clicking OK
    CGUICtrlGeneralDialog* dlg = (CGUICtrlGeneralDialog*)obj;
    
    inputDev.zoom_control = 1+ dlg->combo_zoom->getCurrentItem();

    // if inputMethod has changed:
    int m = dlg->combo_inputMethod->getCurrentItem();
    if (TXInterface->inputMethod() != m)
    {
      cfgfile->setAttributeOverwrite("inputMethod.method", InputMethodStrings[m]);
      std::string msg = reconfigureInputMethod();
      if (msg.length())
        Global::gui->errorMsg(msg.c_str());
    }
    
    // There is no need to restart the TXInterface if only the radio type
    // has changed.
    //~ TXInterface->setRadioType(dlg->combo_radiotype->getCurrentItem());
  }
  
  puDeleteObject(obj);
}

/**
 * Callback for changed input method. Used to show/hide other
 * widgets and check whether some method is available anyway.
 */
static void CGUIInputMethodCallback(puObject *obj)
{
  CGUICtrlGeneralDialog* dlg   = (CGUICtrlGeneralDialog*)obj->getUserData();
  int                    nItem = dlg->combo_inputMethod->getCurrentItem();
  
  if (nItem == T_TX_Interface::eIM_joystick)
  {
    // Is there a joystick available?
    if (SDL_NumJoysticks())
    {
      dlg->combo_joyNum->reveal();
    }
    else
    {
#if TEST_WITHOUT_JOYSTICK > 0
      dlg->combo_joyNum->reveal();
#else     
      Global::gui->errorMsg("No joysticks available");
      dlg->combo_inputMethod->setCurrentItem(T_TX_Interface::eIM_mouse);
      dlg->combo_joyNum->hide();
#endif
    }
  }
  else
  {
    dlg->combo_joyNum->hide();
  }
  
  // value might have been changed above, re-fetch from widget
  nItem = dlg->combo_inputMethod->getCurrentItem();

  // if inputMethod has changed:
  if (TXInterface->inputMethod() != nItem)
  {
    cfgfile->setAttributeOverwrite("inputMethod.method", InputMethodStrings[nItem]);
    std::string msg = reconfigureInputMethod();
    if (msg.length())
      Global::gui->errorMsg(msg.c_str());
  }
  
  if (TXInterface->usesMapper())
  {
    dlg->button_calibrate->activate();
  }
  else
  {
    dlg->button_calibrate->greyOut();
  }

  if (TXInterface->usesMixer())
  {
    dlg->button_mixer->activate();
  }
  else
  {
    dlg->button_mixer->greyOut();
  }
  
  if ((nItem == T_TX_Interface::eIM_joystick)
        ||
       (nItem == T_TX_Interface::eIM_mouse))
  {
    dlg->button_joybuttons->activate();
  }
  else
  {
    dlg->button_joybuttons->greyOut();
  }
  
  if (nItem == T_TX_Interface::eIM_serpic)
  {
    dlg->combo_device->reveal();
    dlg->combo_speed->reveal();
    dlg->combo_device->setCurrentItem(TXInterface->getDeviceNumber());
    dlg->combo_speed->setCurrentItem(TXInterface->getDeviceSpeed());
  }
  else
  {
    dlg->combo_device->hide();
    dlg->combo_speed->hide();
  }

}

/**
 * Callback activated by the Calibrate button
 */
static void CGUICalibrateButtonCallback(puObject *obj)
{
  new CGUICalibMapDialog();
}

/**
 * Callback activated by the "Buttons" button
 */
static void CGUIJoystickButtonCallback(puObject *obj)
{
  CGUICtrlGeneralDialog* dlg   = (CGUICtrlGeneralDialog*)obj->getUserData();
  int                    nItem = dlg->combo_inputMethod->getCurrentItem();

  if (nItem == T_TX_Interface::eIM_mouse)
  {
    new CGUIMouseButtonDialog();
  }
  else
  {
    if (!nJoystickDlg)
    {
      joystickDlg  = new CGUIJoystickDialog();
      nJoystickDlg = 1;
    }
  }
}


/**
 * Callback activated when a different joystick is selected
 */
static void CGUIJoystickCallback(puObject *obj)
{
  puComboBox *combo = static_cast<puComboBox*>(obj);
  int new_joy = combo->getCurrentItem();
  
  if (inputDev.joystick_n != new_joy)
  {
    inputDev.openJoystick(new_joy);
  }
}


/**
 * Callback activated by the "Mixer" button
 */
static void CGUIMixerButtonCallback(puObject *obj)
{
  new CGUIInputScaleDialog(TXInterface);
}

/**
 * Callback activated by the "device" combo box
 */
static void CGUIDeviceCallback(puObject *obj)
{
  puComboBox *cdevice = static_cast<puComboBox*>(obj);
  
  TXInterface->setDeviceNumber(cdevice->getCurrentItem());
  TXInterface->putBackIntoCfg(cfgfile);
  reconfigureInputMethod();
}

/**
 * Callback activated by the "speed" combo box
 */
static void CGUISpeedCallback(puObject *obj)
{
  puComboBox *cspeed = static_cast<puComboBox*>(obj);

  TXInterface->setDeviceSpeed(cspeed->getCurrentItem());
  TXInterface->putBackIntoCfg(cfgfile);
  reconfigureInputMethod();
}
