// implementation of class CGUIMsgBox

#include "crrc_msgbox.h"

#include <iostream>

static void CGUIMsgBoxCallback(puObject *obj);

/** \brief Create the message box
 *
 *  This constructor creates a dialog box which consists of
 *  the message string <code>msg</code> and an OK button.
 *  \param msg The message to be displayed in the box.
 */
CGUIMsgBox::CGUIMsgBox(const char *msg,
                       int buttons,
                       void (*selectCallback)(int))
            : CRRCDialog(0, 0, buttons), selCallback(selectCallback)
{
  // calculate sizes and position
  int msg_width, msg_height;
  int total_width, total_height;

  msg_width = puGetDefaultLegendFont().getStringWidth(msg)
              + PUSTR_LGAP + PUSTR_RGAP;
  msg_height = puGetDefaultLegendFont().getStringHeight(msg)
               + puGetDefaultLegendFont().getStringDescender()
               + PUSTR_TGAP + PUSTR_BGAP;
  //std::cout << "msg_width " << msg_width << std::endl;
  //std::cout << "msg_height " << msg_height << std::endl;

  // 10 px space, string, 10 pixel space, 25 px button, 10 px space
  total_height = DLG_DEF_SPACE + msg_height + DLG_DEF_SPACE
                 + DLG_DEF_BUTTON_HEIGHT + DLG_DEF_SPACE;
  total_width = (msg_width < DLG_DEF_BUTTON_WIDTH ? DLG_DEF_BUTTON_WIDTH : msg_width)
                + 2 * DLG_DEF_SPACE;

  // Create a text widget
  puText *txt = new puText(total_width/2 - msg_width/2,
                           total_height - DLG_DEF_SPACE);
  myMessage = msg;
  txt->setLabel(myMessage.c_str());

  // no more widgets, close the underlying puDialogBox
  close();

  // set the dialog's callback
  setCallback(CGUIMsgBoxCallback);

  // resize the dialog
  int current_width = getABox()->max[0] - getABox()->min[0];
  int current_height = getABox()->max[1] - getABox()->min[1];
  current_width = (current_width > total_width) ? current_width:total_width;
  current_height += msg_height + DLG_DEF_SPACE;
  setSize(current_width, current_height);

  // center the dialog on screen
  int wwidth, wheight;
  puGetWindowSize(&wwidth, &wheight);
  setPosition(wwidth/2 - current_width/2, wheight*2/3 - current_height);

  // show the dialog
  reveal();
}

/** \brief Set a callback.
 *
 *  This is only needed if you construct a box with
 *  more than one button, although it will only work
 *  for a one-button-box.
 */
void CGUIMsgBox::setSelectCallback(void (*cb)(int))
{
  selCallback = cb;
}

/** \brief Invoke the custom callback.
 *
 *  Invokes the box's callback (if a callback was
 *  configured).
 */
void CGUIMsgBox::activateCallback()
{
  if (selCallback != NULL)
  {
    selCallback(getIntegerValue());
  }
}

static void CGUIMsgBoxCallback(puObject *obj)
{
  CGUIMsgBox *dlg = (CGUIMsgBox*)obj;

  dlg->activateCallback();
  puDeleteObject(obj);
}
