// Created 11/09/06 Todd R. Templeton <ttemplet@eecs.berkeley.edu>
// Based on fdm_larcsim.h

#ifndef FDM_DISPLAYMODE_H
#define FDM_DISPLAYMODE_H

#include "../fdm_larcsim/fdm_larcsim.h"
#include "../chardevice.h"

#define DATA_PACKET_SIZE 70

/**
 * 
 * Jens Wilhelm Wulf, September 2005: attempt to (re)structure and simplify LaRCSim code.
 *
 * @author Jens Wilhelm Wulf
 */
class CRRC_AirplaneSim_DisplayMode : public CRRC_AirplaneSim_Larcsim
{
   friend class CRRC_FDM;
   
  public:
   /**
    * Used for sound calculation. It returns the prop's number of revolutions
    * per second [1/s].
    */
   virtual double getPropFreq() { return(0.0); }

   /**
    * Returns relative battery capacity/fuel left (0..1).
    */
   virtual double getBatCapLeft() { return(CRRC_AirplaneSim::getBatCapLeft()); };

   /**
    * Returns velocity relative to airmass [ft/s].
    */
   virtual double getVRelAirmass() { return(0.0); };
   
  private:
   void update(TSimInputs* inputs,
               double      dt,
               int         multiloop);
               
   void update(float*      rawinputs,
               int         n,
               double      dt,
               int         multiloop);

   /**
    * read from file
    */
   CRRC_AirplaneSim_DisplayMode(const char* filename);

   /**
    * read from xml description
    */
   CRRC_AirplaneSim_DisplayMode(SimpleXMLTransfer* xml);

   virtual ~CRRC_AirplaneSim_DisplayMode();
   
   
   void decodePacket(char* buf);
   
   CharDeviceWrapper* io;
   char buf[DATA_PACKET_SIZE];
   int nBytes;
   bool originInitialized;
   double origin[2];
};

#endif
