#include <iostream>

#include "quaternion.cpp"
#include "vector3.cpp"
#include "matrix33.cpp"
#include "intgr.cpp"

/**
 * Quaternion test.
 * 
 * velocity = sin(x)*sin(x)
 * angle    = integral(velocity) dt
 *          = integral(sin(x)*sin(x)) dt
 *          = -0.5 cos(x) sin(x) + 0.5 x
 *              
 */
int main()
{
  const int                   nSteps = 20;
  CRRCMath::Quaternion_002    q1;
  double                      dt = M_PI/((double)nSteps);
  double                      val;
  double                      t;
  double                      soll;
  double                      soll_ende = 0;
  double                      soll_ende_tmp;
  int                         zeit = 0;
  CRRCMath::Vector3           achse[3];

  achse[0] = CRRCMath::Vector3(1, 0, 0);
  achse[1] = CRRCMath::Vector3(0, 1, 0);
  achse[2] = CRRCMath::Vector3(0, 0, 1);

  q1.init(CRRCMath::Vector3());

  for (int a=0; a<3; a++)
  {
    for (int c=0; c<8; c++)
    {
      for (int n=1; n<=nSteps; n++)
      {
        t = M_PI*n/((double)nSteps);

        soll = soll_ende - 0.5*cos(t)*sin(t) + 0.5*t;
        soll_ende_tmp = soll;
        val  = sin(t)*sin(t);

        q1.step(dt, achse[a] * val);

        q1.updateEuler();

        std::cout << zeit++ << " " << soll << " " << q1.euler.r[0] << " " << q1.euler.r[1] << " " << q1.euler.r[2] << " ";
        q1.mat.printLine(); 
        std::cout << "\n";
      }
      soll_ende = soll_ende_tmp;
    }

    for (int c=0; c<8; c++)
    {
      for (int n=1; n<=nSteps; n++)
      {
        t = M_PI*n/((double)nSteps);

        soll = soll_ende + 0.5*cos(t)*sin(t) - 0.5*t;
        soll_ende_tmp = soll;
        val  = sin(t)*sin(t);

        q1.step(dt, achse[a] * -1 * val);

        q1.updateEuler();

        std::cout << zeit++ << " " << soll << " " << q1.euler.r[0] << " " << q1.euler.r[1] << " " << q1.euler.r[2] << " ";
        q1.mat.printLine(); 
        std::cout << "\n";
      }
      soll_ende = soll_ende_tmp;
    }
    
  }
  return(0);
}
